@@ -33,6 +33,7 @@ from ember_csi import constants
 from ember_csi import defaults
 
 
+CONF = config.CONF
 LOG = logging.getLogger(__name__)
 
 
@@ -77,7 +78,7 @@ class IdentityBase(object):
         if self.manifest is not None:
             return
 
-        self.csi_version = version.StrictVersion(config.CSI_SPEC)
+        self.csi_version = version.StrictVersion(CONF.CSI_SPEC)
         self.PLUGIN_CAPABILITIES.append(
             self.TYPES.ServiceType.CONTROLLER_SERVICE)
         caps = [self.TYPES.Capability(service=self.TYPES.Service(type=t))
@@ -101,7 +102,7 @@ class IdentityBase(object):
             manifest['cinder-driver-supported'] = str(self.backend.supported)
 
         self.INFO = self.TYPES.InfoResp(
-            name=config.PLUGIN_NAME,
+            name=CONF.PLUGIN_NAME,
             vendor_version=constants.VENDOR_VERSION,
             manifest=manifest)
         # NOTE(geguileo): For now let's only support single reader/writer modes
@@ -112,7 +113,7 @@ class IdentityBase(object):
         self.CSI.add_IdentityServicer_to_server(self, server)
         self.manifest = manifest
         self.PROBE_KV = cinderlib.objects.KeyValue('%s-%s-%s' % (
-            config.PLUGIN_NAME, config.MODE, 'probe'), '0')
+            CONF.PLUGIN_NAME, CONF.MODE, 'probe'), '0')
 
     def _unsupported_mode(self, capability):
         return capability.access_mode.mode not in self.SUPPORTED_ACCESS
@@ -121,7 +122,7 @@ class IdentityBase(object):
         # TODO: validate mount_flags
         return (capability.HasField('mount') and
                 capability.mount.fs_type and
-                capability.mount.fs_type not in config.SUPPORTED_FS_TYPES)
+                capability.mount.fs_type not in CONF.SUPPORTED_FS_TYPES)
 
     def _validate_capabilities(self, capabilities, context=None):
         msg = ''
@@ -143,7 +144,7 @@ class IdentityBase(object):
 
     def _validate_name(self, name):
         if name and re.match(r'^[A-Za-z]{2,6}(\.[A-Za-z0-9-]{1,63})+$', name):
-                return name
+            return name
 
         return defaults.NAME
 
@@ -162,8 +163,8 @@ class IdentityBase(object):
     def Probe(self, request, context):
         # Proving may take a couple of seconds, and attacher sidecar prior to
         # v0.4 will fail due to small timeout.
-        if self.csi_version < '1.0.0':
-            return self.PROBE_RESP
+        if not CONF.ENABLE_PROBE:
+            return self.TYPES.ProbeRespOK
 
         try:
             self.PROBE_KV.value = str((int(self.PROBE_KV.value) + 1) % 1000)
@@ -190,7 +191,7 @@ class IdentityBase(object):
                 context.abort(grpc.StatusCode.FAILED_PRECONDITION,
                               'Driver failed to return the stats')
 
-        return self.PROBE_RESP
+        return self.TYPES.ProbeRespOK
 
     def _get_vol(self, volume_id=None, always_list=False, **filters):
         backend_name = self.backend.id if self.backend else None
@@ -348,7 +349,8 @@ class ControllerBase(IdentityBase):
             context.abort(grpc.StatusCode.ABORTED,
                           'Operation pending for volume (%s)' % vol.status)
 
-        return self._convert_volume_type(vol)
+        volume = self._convert_volume_type(vol)
+        return self.TYPES.CreateResp(volume=volume)
 
     def _convert_volume_params(self, request, context):
         params = {'qos_specs': {}, 'extra_specs': {}}
@@ -424,16 +426,6 @@ class ControllerBase(IdentityBase):
                 if conn.attached_host != request.node_id:
                     context.abort(grpc.StatusCode.FAILED_PRECONDITION,
                                   'Volume published to another node')
-
-            mode = request.volume_capability.access_mode.mode
-            if ((not request.readonly and
-                 mode == self.TYPES.AccessModeType.SINGLE_NODE_READER_ONLY) or
-                    (request.readonly and
-                     mode == self.TYPES.AccessModeType.SINGLE_NODE_WRITER)):  # noqa
-
-                context.abort(grpc.StatusCode.ALREADY_EXISTS,
-                              'Readonly incompatible with volume capability')
-
         else:
             vol.connect(node.connector_dict, attached_host=node.id)
         return self.TYPES.CtrlPublishResp()
@@ -486,7 +478,8 @@ class ControllerBase(IdentityBase):
         selected, token = self._paginate(request, context, vols)
 
         # TODO(geguileo): Once we support volume types set attributes
-        entries = [self._convert_volume_type(vol) for vol in selected]
+        entries = [self.TYPES.Entry(volume=self._convert_volume_type(vol))
+                   for vol in selected]
         fields = {'entries': entries}
         if token:
             fields['next_token'] = token
@@ -667,7 +660,7 @@ class NodeBase(IdentityBase):
             if not self._check_mount_exists(request.volume_capability,
                                             private_bind, target, context):
                 fs_type = (request.volume_capability.mount.fs_type or
-                           config.DEFAULT_MOUNT_FS)
+                           CONF.DEFAULT_MOUNT_FS)
                 self._format_device(fs_type, private_bind, context)
                 self._mount(fs_type,
                             request.volume_capability.mount.mount_flags,
@@ -748,7 +741,8 @@ class NodeBase(IdentityBase):
         # TODO(geguileo): Check how many are mounted and fail if > 0
 
         # If not published bind it
-        self.sudo('mount', '--bind', staging_target, target)
+        mount_options = 'bind,ro' if request.readonly else 'bind'
+        self.sudo('mount', '-o', mount_options, staging_target, target)
         return self.NODE_PUBLISH_RESP
 
     @common.debuggable
@@ -768,72 +762,42 @@ class NodeBase(IdentityBase):
 
 
 class TopologyBase(object):
-    TOPOLOGIES = None
-    # These 3 attributes are defined in __init__ if we have topologies
-    # TOPOLOGY_HIERA = None
-    # TOPOLOGY_LEVELS = None
-    # TOPOLOGY_LEVELS_SET = None
+    GRPC_TOPOLOGIES = None
+    TOPOLOGY_HIERA = None
 
     def _init_topology(self, constraint_type):
-        if config.TOPOLOGIES:
-            self.TOPOLOGY_HIERA = []
-            self.TOPOLOGIES = []
-            self.TOPOLOGY_LEVELS = []
+        if CONF.TOPOLOGIES:
             if constraint_type not in self.PLUGIN_CAPABILITIES:
                 self.PLUGIN_CAPABILITIES.append(constraint_type)
 
-            for topology in config.TOPOLOGIES:
+            hiera = {}
+            grpc_topos = []
 
-                topo = tuple((k.lower(), v) for k, v in topology.items())
-                topology = self.TYPES.Topology(segments=topology)
+            for topology in CONF.TOPOLOGIES:
+                level = hiera
+                for segment_name, segment_value in topology.items():
+                    value = level.setdefault(segment_name, {})
+                    level = value.setdefault(segment_value, {})
+                grpc_topos.append(self.TYPES.Topology(segments=topology))
 
-                if len(topo) >= len(self.TOPOLOGY_LEVELS):
-                    for k, v in topo[len(self.TOPOLOGY_LEVELS):]:
-                        self.TOPOLOGY_LEVELS.append(k)
-
-                replace = None
-                for i, t in enumerate(self.TOPOLOGY_HIERA):
-                    if t[:len(topo)] == topo:
-                        LOG.warn('Ignoring topology %s. Is included in %s' % (
-                            t, topo))
-                        replace = i
-                        break
-                    elif topo[:len(t)] == t:
-                        LOG.warn('Ignoring topology %s. Is included in %s' % (
-                            topo, t))
-                        break
-                else:
-                    self.TOPOLOGY_HIERA.append(topo)
-                    self.TOPOLOGIES.append(topology)
-
-                if replace is not None:
-                    self.TOPOLOGY_HIERA[i] = topo
-                    self.TOPOLOGIES[i] = topology
-
-            LOG.debug("topology: %s" % self.TOPOLOGY_HIERA)
-            self.TOPOLOGY_LEVELS_SET = set(self.TOPOLOGY_LEVELS)
+            self.TOPOLOGY_HIERA = hiera
+            self.GRPC_TOPOLOGIES = grpc_topos
 
     def _topology_is_accessible(self, topology, context):
-        topo = []
-        unused_domains = set(topology.keys())
+        unchecked = list(topology.segments.keys())
+        level = self.TOPOLOGY_HIERA
+        while level and unchecked:
+            for segment_name in level:
+                if segment_name in unchecked:
+                    value = topology.segments[segment_name]
+                    level = level[segment_name].get(value)
+                    unchecked.remove(segment_name)
+                    break
 
-        for domain in self.TOPOLOGY_LEVELS:
-            if domain not in topology:
-                break
-            topo.append((domain, topology[domain]))
-            unused_domains.remove(domain)
-
-        # We used the domains in hierarchical order, if there is any known
-        # domain in the request we haven't used, then there was one level that
-        # was missing.
-        if unused_domains.intersection(self.TOPOLOGY_LEVELS_SET):
-            context.abort(grpc.StatusCode.INVALID_ARGUMENT,
-                          'Missing domain topology in request.')
-
-        for t in self.TOPOLOGY_HIERA:
-            if topo == t[:len(topo)]:
-                return True
-        return False
+        # Accessible if value of a segment name didn't fail, and if one of the
+        # topologies is a subset of the other one.
+        return level is not None and not (unchecked and level)
+        # return not (levels is None or (unchecked_segments and level))
 
     def _validate_accessible_requirements(self, topology_req, context):
         requisite = getattr(topology_req, 'requisite', None)
@@ -850,15 +814,20 @@ class TopologyBase(object):
                                   'All preferred topologies must be in '
                                   'requisite topologies')
 
-        to_check = requisite or preferred
-        for topology in to_check:
+        # If we only have preferred, we can ignore it, after all we don't have
+        # different topologies to choose from.
+        if not requisite:
+            return
+
+        for topology in requisite:
             if self._topology_is_accessible(topology, context):
                 return
+
         context.abort(grpc.StatusCode.INVALID_ARGUMENT,
                       'None of the requested topologies are accessible.')
 
     def _validate_accessibility(self, request, context):
-        if not self.TOPOLOGIES:
+        if not self.GRPC_TOPOLOGIES:
             return
 
         # Used by CreateVolume
